@@ -92,6 +92,8 @@ class DataFixture(object):
     :field: Field object.
     :key: string that represents a unique name for a Field, considering app, model and field names.
     """
+    def __init__(self):
+        self.plugins = {}
 
     def _field_fixture_template(self, field_class):
         return '%s_config' % (field_class.__name__.lower(),)
@@ -112,6 +114,12 @@ class DataFixture(object):
 
     def generate_data(self, field):
         "Get a unique and valid data for the field."
+        field_fullname = field.__module__ + "." + field.__class__.__name__
+        fixture = self.plugins.get(field_fullname, {})
+        if type(fixture) == dict:
+            fixture = fixture.get('ddf_fixture', None)
+        if fixture and callable(fixture):
+            return fixture()
         config = self._field_fixture_factory(field.__class__)
         is_supported_field = config != None
         if is_supported_field:
@@ -439,7 +447,7 @@ class DynamicFixture(object):
             raise InvalidModelError(get_unique_model_name(model_class))
         for field in get_fields_from_model(model_class):
             if is_key_field(field) and 'id' not in configuration: continue
-            if field.name in self.ignore_fields: continue
+            if field.name in self.ignore_fields and field.name not in self.kwargs: continue
             self.set_data_for_a_field(model_class, instance, field, persist_dependencies=persist_dependencies, **configuration)
         number_of_pending_fields = len(self.pending_fields)
         # For Copier fixtures: dealing with pending fields that need to receive values of another fields.
