@@ -42,6 +42,7 @@ class AutoDataFiller(object):
 class SequentialDataFixture(DataFixture):
 
     def __init__(self):
+        super(SequentialDataFixture, self).__init__()
         self.filler = AutoDataFiller()
 
     def get_value(self, field, key):
@@ -142,6 +143,10 @@ class SequentialDataFixture(DataFixture):
     def imagefield_config(self, field, key):
         return six.text_type(self.get_value(field, key))
 
+    # BINARY
+    def binaryfield_config(self, field, key):
+        return six.b(self.charfield_config(field, key))
+
 
 class GlobalSequentialDataFixture(SequentialDataFixture):
     def get_value(self, field, key):
